@@ -4,7 +4,7 @@ use 5.006_001;
 use strict;
 use warnings;
 
-our $VERSION = '0.11';
+our $VERSION = '0.20';
 
 use Carp;
 use File::Basename;
@@ -15,19 +15,23 @@ require Exporter;
 our @ISA = qw(Exporter);
 
 our %EXPORT_TAGS = ( 'all' => [ qw(
-	create_desktop_shortcut_filename
-	create_url_shortcut_filename
-	create_webloc_shortcut_filename
-	write_desktop_shortcut_file
-	write_url_shortcut_file
-	write_webloc_binary_shortcut_file
-	write_webloc_xml_shortcut_file
+    create_desktop_shortcut_filename
+    create_url_shortcut_filename
+    create_webloc_shortcut_filename
+    write_desktop_shortcut_file
+    write_url_shortcut_file
+    write_webloc_binary_shortcut_file
+    write_webloc_xml_shortcut_file
+    write_desktop_shortcut_handle
+    write_url_shortcut_handle
+    write_webloc_binary_shortcut_handle
+    write_webloc_xml_shortcut_handle
 ) ] );
 
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 
 our @EXPORT = qw(
-	
+
 );
 
 
@@ -126,7 +130,7 @@ my $default_max_filename_length = 100;
 
 sub _create_filename {
     my ($name, $length, $extension) = @_;
-    
+
     if(!defined($length)) {
         $length = $default_max_filename_length;
     } else {
@@ -135,11 +139,11 @@ sub _create_filename {
             croak("Length parameter must be greater than or equal to ${min_length}")
         }
     }
-    
+
     if(!defined($name)) {
         $name = "";
     }
-    
+
     my $max_basename_length = $length - length($extension);
 
     # The valid characters are listed below in ASCII order.
@@ -152,26 +156,26 @@ sub _create_filename {
     }
 
     my $filename = substr($clean_name, 0, $max_basename_length) . $extension;
-    
+
     return $filename;
 }
 
 # $length Includes file name and extension (no path).
 sub create_desktop_shortcut_filename {
-	my ($name, $length) = @_;
-	
-	_create_filename($name, $length, $desktop_extension);
+    my ($name, $length) = @_;
+
+    _create_filename($name, $length, $desktop_extension);
 }
 
 sub create_url_shortcut_filename {
     my ($name, $length) = @_;
-    
+
     _create_filename($name, $length, $url_extension);
 }
 
 sub create_webloc_shortcut_filename {
     my ($name, $length) = @_;
-    
+
     _create_filename($name, $length, $webloc_extension);
 }
 
@@ -180,9 +184,9 @@ sub create_webloc_shortcut_filename {
 ### The writers
 
 sub _check_file_already_exists {
-	my ( $filename ) = @_;
-	
-	if(-e $filename) {
+    my ( $filename ) = @_;
+
+    if(-e $filename) {
         croak "File ${filename} already exists";
     }
 }
@@ -216,82 +220,133 @@ must be installed in order to write ".webloc" files.
 
 sub write_desktop_shortcut_file {
     my ( $filename, $name, $url ) = @_;
- 
+
     _check_file_already_exists ( $filename );
     open (my $file, ">:encoding(UTF-8)", $filename) or die "Error opening file \"${filename}\": $!";
-    
-    # Assume all the writes will be done in UTF-8.
-    print $file "[Desktop Entry]\n";
-    print $file "Encoding=UTF-8\n";
-    print $file "Name=${name}\n";
-    print $file "Type=Link\n";
-    print $file "URL=${url}\n";
-    
+
+    write_desktop_shortcut_handle($file, $name, $url);
+
     close ($file);
-    
+
     return 1;
 }
 
 sub write_url_shortcut_file {
     my ( $filename, $name, $url ) = @_;
- 
+
     _check_file_already_exists ( $filename );
     open (my $file, ">", $filename) or die "Error opening file \"${filename}\": $!";
-    
-    if(is_utf8($url)) {
-        print $file "[InternetShortcut.W]\r\n";
-        my $url_utf7 = encode("UTF-7", $url);
-        print $file "URL=${url_utf7}\r\n";    	
-    } else {
-        print $file "[InternetShortcut]\r\n";
-        print $file "URL=${url}\r\n";
-    }
-    
+
+    write_url_shortcut_handle($file, $name, $url);
+
     close ($file);
-    
+
     return 1;
 }
 
-# TODO: Fix this eval to not use an expression.  This causes it to fail perlcritic.
-sub _try_load_module_for_webloc {
-	my ( $module, $list ) = @_;
-
-    eval ( "use ${module} ${list}; 1" ) or
-        die "Could not load ${module} module.  This module is required in order to read/write webloc files.  Error: $@";
-}
-
 sub write_webloc_binary_shortcut_file {
     my ( $filename, $name, $url ) = @_;
- 
-    _try_load_module_for_webloc ( "Mac::PropertyList", "qw(:all)" );
-    _try_load_module_for_webloc ( "Mac::PropertyList::WriteBinary", "" );
-  
+
     _check_file_already_exists ( $filename );
 
-    my $data = new Mac::PropertyList::dict({ "URL" => $url });
-    my $buf = Mac::PropertyList::WriteBinary::as_string($data);
-    
     open (my $file, ">:encoding(UTF-8)", $filename) or die "Error opening file \"${filename}\": $!";
     binmode $file;
-    print $file $buf;
+    write_webloc_binary_shortcut_handle($file, $name, $url);
     close ($file);
-    
+
     return 1;
 }
 
 sub write_webloc_xml_shortcut_file {
     my ( $filename, $name, $url ) = @_;
- 
-    _try_load_module_for_webloc ( "Mac::PropertyList", "qw(:all)" );
- 
+
     _check_file_already_exists ( $filename );
-    
-    my $str = create_from_hash({ "URL" => $url });
-    
+
     open (my $file, ">:encoding(UTF-8)", $filename) or die "Error opening file \"${filename}\": $!";
-    print $file $str;
+    write_webloc_xml_shortcut_handle($file, $name, $url);
     close ($file);
-    
+
+    return 1;
+}
+
+
+
+=item write_desktop_shortcut_handle( HANDLE, NAME, URL )
+
+=item write_url_shortcut_handle( HANDLE, NAME, URL )
+
+=item write_webloc_binary_shortcut_handle( HANDLE, NAME, URL )
+
+=item write_webloc_xml_shortcut_handle( HANDLE, NAME, URL )
+
+Similar to the corresponding file writers, but writes to an
+IO::Handle object instead.
+
+=cut
+
+sub write_desktop_shortcut_handle {
+    my ( $handle, $name, $url ) = @_;
+
+    # Assume all the writes will be done in UTF-8.
+    print $handle "[Desktop Entry]\n";
+    print $handle "Encoding=UTF-8\n";
+    print $handle "Name=${name}\n";
+    print $handle "Type=Link\n";
+    print $handle "URL=${url}\n";
+
+    close ($handle);
+
+    return 1;
+}
+
+sub write_url_shortcut_handle {
+    my ( $handle, $name, $url ) = @_;
+
+    if(is_utf8($url)) {
+        print $handle "[InternetShortcut.W]\r\n";
+        my $url_utf7 = encode("UTF-7", $url);
+        print $handle "URL=${url_utf7}\r\n";
+    } else {
+        print $handle "[InternetShortcut]\r\n";
+        print $handle "URL=${url}\r\n";
+    }
+
+    close ($handle);
+
+    return 1;
+}
+
+# TODO: Fix this eval to not use an expression.  This causes it to fail perlcritic.
+sub _try_load_module_for_webloc {
+    my ( $module, $list ) = @_;
+
+    eval ( "use ${module} ${list}; 1" ) or
+        die "Could not load ${module} module.  This module is required in order to read/write webloc files.  Error: $@";
+}
+
+sub write_webloc_binary_shortcut_handle {
+    my ( $handle, $name, $url ) = @_;
+
+    _try_load_module_for_webloc ( "Mac::PropertyList", "qw(:all)" );
+    _try_load_module_for_webloc ( "Mac::PropertyList::WriteBinary", "" );
+
+    my $data = new Mac::PropertyList::dict({ "URL" => $url });
+    my $buf = Mac::PropertyList::WriteBinary::as_string($data);
+    print $handle $buf;
+    close ($handle);
+
+    return 1;
+}
+
+sub write_webloc_xml_shortcut_handle {
+    my ( $handle, $name, $url ) = @_;
+
+    _try_load_module_for_webloc ( "Mac::PropertyList", "qw(:all)" );
+
+    my $str = create_from_hash({ "URL" => $url });
+    print $handle $str;
+    close ($handle);
+
     return 1;
 }
 
@@ -313,4 +368,3 @@ This library is free software; you can redistribute it and/or modify
 it under the same terms as the Perl 5 programming language itself.
 
 =cut
-
